//
//  main.c
//  Example using MFS as UART
//
//  Created by Manuel Schreiner on 17.04.16,
//  Copyright © 2016 io-expert.com. All rights reserved.
//
/*******************************************************************************/
//                                                                             //
//   IIIII    OOO       EEE  X  X  PPP  EEE  RRR  TTTTT       CCC  OOO  M   M  //
//  I     I  O   O      E    X  X  P  P E    R  R   T        C    O   O MM MM  //
//   I M I   O   O  --  EEE   XX   PPP  EEE  RRR    T        C    O   O M M M  //
//    III    O   O      E    X  X  P    E    R  R   T        C    O   O M   M  //
//    III     OOO       EEE  X  X  P    EEE  R  R   T   DOT   CCC  OOO  M   M  //
//                                                                             //
/*******************************************************************************/
//
// 1. Redistributions of source code must retain the above copyright notice,
//    this condition and the following disclaimer.
//
// This software is provided by the copyright holder and contributors "AS IS"
// and any warranties related to this software are DISCLAIMED.
// The copyright owner or contributors be NOT LIABLE for any damages caused
// by use of this software.
//
/*******************************************************************************/
/** \file main.c
 **
 ** Example using MFS as UART
 **
 ** Main Module
 **
 ** History:
 **   - 2016-04-17  First Version
 **
 *******************************************************************************/
#include "mcu.h"
#include "base_types.h"

/**
 ******************************************************************************
 ** \brief  UART Baudrate calculation
 **
 ** <br>    UART Baudrate: BGR = freq_peripheral_bus/baudrate - 1              
 ** <br>    The MFS is connected at peripheral bus 1 for FM0+, 2 for FM3 and FM4                
 ** <br>    APBCx_PSR_Val is the peripheral divider defined in system_<device>.h                 
 ** <br>    __HCLK is determined by system_<device>.h and represents           
 ** <br>    the clock before the peripheral divider 
 **
 ** \param  x baudrate
 **
 ** \return BGR value
 **
 *****************************************************************************/
#if defined(FM0P_PERIPH_BASE)
    #define UART_BAUDRATE(x)  (uint16_t)((__HCLK / ((APBC1_PSR_Val & 0x03) + 1))/((uint32_t)x - 1))
#elif defined(FM4_PERIPH_BASE) || defined(FM3_PERIPH_BASE)
    #define UART_BAUDRATE(x)  (uint16_t)((__HCLK / ((APBC2_PSR_Val & 0x03) + 1))/((uint32_t)x - 1))      
#endif

/**
 ******************************************************************************
 ** \brief  initialize UART
 **
 ** \param  pstcUart  UART pointer
 **
 *****************************************************************************/
void InitUart( FM_MFS_UART_TypeDef* pstcUart )
{
  //DDR0 |=0x1000;
  pstcUart->SCR  = 0x80;  //set UPCL (bit 7) to reset the UART
  pstcUart->SMR  = 0x01;  //set MFS to UART mode MD0..3 to 1 (bit 0..3)
  pstcUart->SSR  = 0x00;  //clear status register
  pstcUart->ESCR = 0x00;  
  pstcUart->BGR = UART_BAUDRATE(115200);
  pstcUart->SCR = 0x03;   //enable TX (bit 0) and RX (bit 1)
}


/**
 ******************************************************************************
 ** \brief  sends a single character  (no timeout !)
 **
 ** \param  pstcUart  UART pointer
 **
 ** \param  u8Char    Data to send
 **
 *****************************************************************************/
void PutCharUart(FM_MFS_UART_TypeDef* pstcUart, uint8_t u8Char)
{
  while (!(pstcUart->SSR & (1 << 1)));   // wait for transmit buffer empty (bit 1)
  pstcUart->TDR = u8Char;                // put u8Char into buffer
    
}


/**
 ******************************************************************************
 ** \brief  sends a complete string (0-terminated) 
 **
 ** \param  pstcUart  UART pointer
 **
 ** \param  Pointer to (constant) file of bytes in mem
 **
 *****************************************************************************/
void PutStringUart(FM_MFS_UART_TypeDef* pstcUart, char_t *pu8Buffer)
{
  while (*pu8Buffer != '\0')
  { 
    PutCharUart(pstcUart,*pu8Buffer++);        // send every char of string
  }
}


/**
 ******************************************************************************
 ** \brief  returns incoming character, if received on UART0
 **
 ** \param  pstcUart  UART pointer
 **
 ** \return Character or 0xFF (Error) or 0 (Nothing)
 *****************************************************************************/
uint8_t GetCharUart(FM_MFS_UART_TypeDef* pstcUart)   
{
  uint8_t u8Char;
  
  if ((pstcUart->SSR & 0x38) != 0)  // Check for errors ORE (bit 3), FRE (bit 4) and PE (bit 5)
  {
    pstcUart->SSR |= (1 << 7);      // Clear error flags by set REC (bit 7) 
    u8Char = pstcUart->RDR;         // Flush RDR
    return 0xFF;
  }
  else if (pstcUart->SSR & 0x04)    // check RDRF (bit 2)
  {
    u8Char = pstcUart->RDR;
    return (u8Char); 
  }
  else
  {
    return 0; 
  }
}

/**
 ******************************************************************************
 ** \brief  Main function of project
 **
 ** \param  none
 **
 ** \return int32_t return value, if needed
 ******************************************************************************/
int32_t main(void)
{
  uint8_t u8Char;
  FM_MFS_UART_TypeDef* pstcUart;
  
  /* Set the used UART here */
  pstcUart = FM_MFS0_UART;  //use UART0
  SetPinFunc_SIN0_0();      //use RX UART0, relocated 0
  SetPinFunc_SOT0_0();      //use TX UART0, relocated 0
  
  // initialize UARTs
  InitUart(pstcUart);
  PutStringUart(pstcUart,"\n\rHello World!\r\n");
  
  for(;;)
  {
    // Repeat UART input
    u8Char = GetCharUart(pstcUart);
    if (u8Char != 0)
    {
      PutCharUart(pstcUart,u8Char);
    }
  } // for(;;)

}
